--
-- sfiforam_v.vhd - Synchronous FIFO module RAM array
--
-- Modules:
--
--    sfiforam       512word by 32bit dual-ported memory using
--                   BlockRAMs for Virtex/Virtex-E/Virtex-EM
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity sfiforam is
    port (
        clk:           in       std_logic;
        rst:           in       std_logic;
        sr:            in       std_logic;
        da:            in       std_logic_vector(8 downto 0);
        d:             in       std_logic_vector(31 downto 0);
        qa:            in       std_logic_vector(8 downto 0);
        q:             out      std_logic_vector(31 downto 0);
        r:             in       std_logic;
        w:             in       std_logic);
end;

architecture virtex of sfiforam is

    signal logic0:    std_logic;
    signal logic1:    std_logic;
    signal ram_zero:  std_logic_vector(7 downto 0);
    
    component RAMB4_S8_S8
        port(
            DOA:        out std_logic_vector(7 downto 0); 
            DOB:        out std_logic_vector(7 downto 0); 
            ADDRA:      in  std_logic_vector(8 downto 0);
            DIA:        in  std_logic_vector(7 downto 0);
            ENA:        in  std_logic;
            CLKA:       in  std_logic;
            WEA:        in  std_logic;
            RSTA:       in  std_logic;
            ADDRB:      in  std_logic_vector(8 downto 0);
            DIB:        in  std_logic_vector(7 downto 0);
            ENB:        in  std_logic;
            CLKB:       in  std_logic;
            WEB:        in  std_logic;
            RSTB:       in  std_logic);
    end component;
    
begin

    logic0 <= '0';
    logic1 <= '1';
    ram_zero <= (others => '0');
    
    gen_virtex : for i in 0 to 3 generate
        ram_virtex: RAMB4_S8_S8
            port map(
                DOA   => open,
                DOB   => q(8 * (i + 1) - 1 downto 8 * i),
                ADDRA => da,
                DIA   => d(8 * (i + 1) - 1 downto 8 * i),
                ENA   => logic1,
                CLKA  => clk,
                WEA   => w,
                RSTA  => sr,
                ADDRB => qa,
                DIB   => ram_zero,
                ENB   => r,
                CLKB  => clk,
                WEB   => logic0,
                RSTB  => sr);
    end generate;

end architecture;

