--
-- sfiforam_v2.vhd - Synchronous FIFO module RAM array
--
-- Modules:
--
--    sfiforam       512word by 32bit dual-ported memory using
--                   BlockRAMs for Virtex-II
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-- synopsys translate_off
library unisim;
use unisim.vcomponents.all;
-- synopsys translate_on

entity sfiforam is
    port (
        clk:           in       std_logic;
        rst:           in       std_logic;
        sr:            in       std_logic;
        da:            in       std_logic_vector(8 downto 0);
        d:             in       std_logic_vector(31 downto 0);
        qa:            in       std_logic_vector(8 downto 0);
        q:             out      std_logic_vector(31 downto 0);
        r:             in       std_logic;
        w:             in       std_logic);
end;

architecture virtex2 of sfiforam is

    signal logic0:    std_logic;
    signal logic1:    std_logic;
    signal ram_zero:  std_logic_vector(31 downto 0);
    
    component RAMB16_S36_S36
        port(
            DOA:        out std_logic_vector(31 downto 0); 
            DOPA:       out std_logic_vector(3 downto 0); 
            DOB:        out std_logic_vector(31 downto 0); 
            DOPB:       out std_logic_vector(3 downto 0); 
            ADDRA:      in  std_logic_vector(8 downto 0);
            DIA:        in  std_logic_vector(31 downto 0);
            DIPA:       in  std_logic_vector(3 downto 0);
            ENA:        in  std_logic;
            CLKA:       in  std_logic;
            WEA:        in  std_logic;
            SSRA:       in  std_logic;
            ADDRB:      in  std_logic_vector(8 downto 0);
            DIB:        in  std_logic_vector(31 downto 0);
            DIPB:       in  std_logic_vector(3 downto 0);
            ENB:        in  std_logic;
            CLKB:       in  std_logic;
            WEB:        in  std_logic;
            SSRB:       in  std_logic);
    end component;
    
    attribute WRITE_MODE_A : string;
    attribute WRITE_MODE_B : string;
    attribute WRITE_MODE_A of ram0 : label is "READ_FIRST";
    attribute WRITE_MODE_B of ram0 : label is "READ_FIRST";

begin

    logic0 <= '0';
    logic1 <= '1';
    ram_zero <= (others => '0');
    
    ram0: RAMB16_S36_S36
        port map(
            DOA   => open,
            DOPA  => open,
            DOB   => q,
            DOPB  => open,
            ADDRA => da,
            DIA   => d,
            DIPA  => ram_zero(3 downto 0),
            ENA   => logic1,
            CLKA  => clk,
            WEA   => w,
            SSRA  => sr,
            ADDRB => qa,
            DIB   => ram_zero,
            DIPB  => ram_zero(3 downto 0),
            ENB   => r,
            CLKB  => clk,
            WEB   => logic0,
            SSRB  => sr);

end architecture;
